/*
 *  cursor.c
 *
 *  CAVE - Colored Ascii Visual Editor
 *  Cursor movement routines
 *
 *  Copyright (C) 2002 Michal Miszewski <fixer@irc.pl>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License version 2 as
 *  published by the Free Software Foundation.
 */

#include <curses.h>
#include <stdlib.h>
#include "config.h"
#include "cursor.h"

int actx = 0, acty = 0;
short mode = 0;  /* cursor movement mode */
chtype oldchar;  /* character under the cursor */

extern struct config
{
  int defcolor, defbgnd, defattr, defmode, fromzero,
  key_up, key_down, key_left, key_right,
  key_leftborder, key_rightborder, key_upborder, key_downborder,
  key_delch, key_backspc, key_addattribs,
  key_export, key_import, key_color, key_bgnd, key_attrib, key_mode;
} conf;

/* puts the cursor on selected position */
void put_cursor(int y, int x)
{
  acty = y;
  actx = x;
  oldchar = mvwinch(editw, y, x);
  mvwaddch(editw, y, x, oldchar | A_REVERSE);
}

/* hides the cursor (for import/export.c routines) */
void cursor_up(void)
{
  mvwaddch(editw, acty, actx, oldchar);
  wrefresh(editw);
}

/* shows the cursor (for import/export.c routines) */
void cursor_down(void)
{
  put_cursor(acty, actx);
  wrefresh(editw);
}

/* moves the cursor to selected position */
void mv_cursor(int y, int x)
{
  mvwaddch(editw, acty, actx, oldchar);  /* restore non-inverted char */
  put_cursor(y, x);
  wrefresh(editw);
}

/* moves the cursor up */
void move_up(void)
{
  if (acty > 0) mv_cursor(acty - 1, actx);
}

/* moves the cursor down */
void move_down(void)
{
  if (acty < (ysize - 1)) mv_cursor(acty + 1, actx);
}

/* moves the cursor left */
void move_left(void)
{
  if (actx > 0) mv_cursor(acty, actx - 1);
}

/* moves the cursor right */
void move_right(void)
{
  if (actx < (xsize - 1)) mv_cursor(acty, actx + 1);
}

#ifdef RANDOM

/* generates random number between min and max */
int randomize(int min, int max)
{
  int ret;

  srandom(fgetc(rnd));
  ret = ((random() % (max - min + 1)) + min);
  return ret;	  
}

#endif

/* steps using method selected in 'mode' */
void step(void)
{
  int one = mode;

#ifdef RANDOM
  if (mode == 9) one = randomize(1, 8);  /* the move_rnd mode */
#endif
  switch (one)
  {
    case 1: { move_right(); break; }
    case 2: { move_left(); break; }
    case 3: { move_up(); break; }
    case 4: { move_down(); break; }
    case 5: { move_up(); move_left(); break; }
    case 6: { move_up(); move_right(); break; }
    case 7: { move_down(); move_left(); break; }
    case 8: { move_down(); move_right(); break; }
  }
}

/* steps backwards using method selected in 'mode' */
void step_bwd(void)
{
  int one = mode;

#ifdef RANDOM
  if (mode == 9) one = randomize(1, 8);  /* the move_rnd mode */
#endif
  switch (one)
  {
    case 1: { move_left(); break; }
    case 2: { move_right(); break; }
    case 3: { move_down(); break; }
    case 4: { move_up(); break; }
    case 5: { move_down(); move_right(); break; }
    case 6: { move_down(); move_left(); break; }
    case 7: { move_up(); move_right(); break; }
    case 8: { move_up(); move_left(); break; }
  }
}

/* gets the start-up values from the conf structure */
void cursor_load_defaults(void)
{
  mode = conf.defmode;
}
