/*
 *  export.c
 *
 *  CAVE - Colored Ascii Visual Editor
 *  Image export routines
 *
 *  Copyright (C) 2002 Michal Miszewski <fixer@irc.pl>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License version 2 as
 *  published by the Free Software Foundation.
 */

#include <curses.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include "export.h"
#include "cursor.h"
#include "color.h"
#include "status.h"

FILE *fd;
extern char lastname[500];

/* opens a file */
int open_file(char filename[], int mode)
{
  int ret = 0;

  if (mode == 1) fd = fopen(filename, "w"); else fd = fopen(filename, "a+");
  if (fd == NULL)  /* error message on status */
  {
    ret = 1;
    status_error();
  }
  return ret;
}

/* finds the real end of line, without unimportant spaces */
int end(char *buf)
{
  int last;

  for (last = strlen(buf) - 1; last >= 0; last--)
    if (buf[last] != ' ') break;
  return last;
}

/* ansi text export */
/* sorry because of the obscure code */
void export_ansi(void)
{
  int x, y, was = 0, waszero;
  short color, bgnd, attr;
  short lastcolor = EDEFCOLOR, lastbgnd = EDEFBGND, lastattr = EDEFATTR;
  chtype ch;
  char sbuf[10];
  char *buf;  /* line buffer */

  buf = (char *)malloc(sizeof(char) * CBUF);
  if (buf == NULL)
  {
    status_message("Error: Can't allocate memory.");
    return;
  }
  for (y = 0; y < ysize; y++)
  {
    strcpy(sbuf, "");
    strcpy(buf, "");
    for (x = 0; x <= xsize; x++)
    {
      if (x != xsize) ch = mvwinch(editw, y, x);
      else  /* we are outside the screen */
      ch = ' ' | A_NORMAL | COLOR_PAIR(find_pair(COLOR_WHITE, COLOR_BLACK));
      /* if something goes wrong */
      if (pair_content(PAIR_NUMBER(ch & A_COLOR), &color, &bgnd) == ERR)
      {
        color = COLOR_WHITE;  // !!! fix them to 'defines'
	bgnd = COLOR_BLACK;   // !!! shouldn't it be chtype ???
      }
      switch (ch & A_ATTRONLY)  /* changes curses attr names into ansi */
      {
        case A_NORMAL:    { attr = 0; break; }
	case A_BOLD:      { attr = 1; break; }
	case A_UNDERLINE: { attr = 4; break; }
	case A_BLINK:     { attr = 5; break; }
	case A_REVERSE:   { attr = 7; break; }
	default:          { attr = 0; }
      }
      was = 0;
      waszero = 0;
      /* if any codes are needed */
      if ((color != lastcolor) || (bgnd != lastbgnd) || (attr != lastattr))
      {
        sprintf(sbuf, "\x1b[");
	strcat(buf, sbuf);
        if (attr != lastattr)  /* the attribute has changed */
        {
	  if (was == 1)
	  {
	    sprintf(sbuf, ";");
	    strcat(buf, sbuf);
	  }
	  if ((lastattr != 0) && (attr != 0))  /* clear all attributes before */
	  {                                    /* setting another one */
	    sprintf(sbuf, "0;");
	    strcat(buf, sbuf);
	    waszero = 1;  /* inform that attributes were cleared */
          }
          sprintf(sbuf, "%d", attr);
          strcat(buf, sbuf);
	  if ((attr == 0) || (waszero != 0))
	  {
	    /* force adding color codes by changing last colors/bgnds to
             * non-existant values */
	    lastcolor = 20;
	    lastbgnd = 20;
	  }
	  was = 1;
        }
        if ((color != lastcolor))  /* the color has changed */
        {
	  if (was == 1)
	  {
	    sprintf(sbuf, ";");
	    strcat(buf, sbuf);
	  }
          sprintf(sbuf, "%d", 30 + color);
          strcat(buf, sbuf);
	  was = 1;
        }
        if ((bgnd != lastbgnd))  /* the background color has changed */
        {
	  if (was == 1)
	  {
	    sprintf(sbuf, ";");
	    strcat(buf, sbuf);
	  }
          sprintf(sbuf, "%d", 40 + bgnd);
          strcat(buf, sbuf);
	  was = 1;
        }
	if (was == 1)  /* the code was generated, must be ended with 'm' */
	{
	  sprintf(sbuf, "m");
          strcat(buf, sbuf);
	}
      }
      if (x != xsize)  /* if we are not outside the screen */
      {
        sprintf(sbuf, "%c", (char)ch);
        strcat(buf, sbuf);
      }
      lastcolor = color;
      lastbgnd = bgnd;
      lastattr = attr;
    }
    /* remove all unneeded spaces */
    buf[end(buf) + 1] = '\0';
    strcpy(sbuf, "");
    /* all attributes off at the end of line ( ESC[0m ) */
    if ((strlen(buf) != 0) && (was != 0))
    {
      sprintf(sbuf, "\x1b[0m");
      strcat(buf, sbuf);
    }
    fputs(buf, fd);
    fputc('\n', fd);
    lastcolor = EDEFCOLOR;
    lastbgnd = EDEFBGND;
    lastattr = EDEFATTR;
  }
  free(buf);
}

/* plain text export */
void export_txt(void)
{
  int x, y;
  char *buf;  /* line buffer */

  buf = (char *)malloc(sizeof(char) * CBUF);
  if (buf == NULL)
  {
    status_message("Error: Can't allocate memory.");
    return;
  }
  for (y = 0; y < ysize; y++)
  {
    for (x = 0; x < xsize; x++) buf[x] = mvwinch(editw, y, x);
    buf[x] = '\0';
    buf[end(buf) + 1] = '\0';
    fputs(buf, fd);
    fputc('\n', fd);
  }
  free(buf);
}

void export(void)
{
  char filename[200] = "";
  char realname[200] = "";
  int format, a, wr = 1;

  format = status_question("Image export. Which format", "ANSI Text Cancel", 1);
  if (format != 3)  /* check this after adding new filters! */
  {
    status_input("File name", lastname, filename, 60);
    if (strncmp(filename, "", 200) != 0)
    {
      for (a = 0; a < strlen(filename); a++)
        if (filename[a] == '~')
	{
	  status_message("~ not allowed; enter full pathname.");
	  return;
	}
      realpath(filename, realname);
      if (access(realname, F_OK) == 0)
        wr = status_question("File Exists. Overwrite", "Yes No Append", 1);
      if (wr != 2)
      {
        if (open_file(realname, wr) == 0)
        {
          cursor_up();
          switch (format)
          {
            case 1: { export_ansi(); break; }
            case 2: { export_txt(); break; }
          }
          fclose(fd);
          strcpy(lastname, filename);
          cursor_down();
        }
      }
    }
  }
}
